@@ -25,6 +25,7 @@
  */
 
 #include <errno.h>
+#include <locale.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <sys/file.h>
@@ -97,7 +98,7 @@ main(argc, argv)
     krb5_creds *my_creds;
     krb5_auth_context auth_context;
 
-    setlocale(LC_MESSAGES, "");
+    setlocale(LC_ALL, "");
     retval = krb5_init_context(&context);
     if (retval) {
         com_err(argv[0], retval, _("while initializing krb5"));
